@@ -232,7 +232,6 @@ SV *Py2Pl(PyObject * const obj) {
 #else
                     key_val = PyString_AsString(s);
 #endif
-                    Py_DECREF(s);
                     if (PL_dowarn)
                         warn("Stringifying non-string hash key value: '%s'",
                              key_val);
@@ -548,7 +547,9 @@ croak_python_exception() {
         PyObject *perl_exception = PySequence_GetItem(perl_exception_args, 0);
         SV *perl_exception_object = Py2Pl(perl_exception);
         sv_2mortal(perl_exception_object);
-        croak_sv(perl_exception_object);
+        SV *errsv = get_sv("@", GV_ADD);
+        sv_setsv(errsv, perl_exception_object);
+        croak(NULL);
         Py_DECREF(perl_exception);
         Py_DECREF(perl_exception_args);
     }
